from distutils.core import setup
setup(
	name = 'transfromage',
	packages = ['transfromage'],
	version = '0.1',
	license = 'MIT',
	description = 'Client library for the Transformice game',
	author = 'Tocutoeltuco - Turkitutu',
	author_email = 'tocutoeltuco@gmail.com',
	url = 'https://github.com/Tocutoeltuco/transfromage',
	download_url = 'https://github.com/Tocutoeltuco/transfromage/archive/v_01.tar.gz',
	keywords = ['TRANFORMICE', 'CLIENT', 'THREADING', 'ATELIER801', 'EVENT'],
	install_requires=[],
	classifiers=[
		'Development Status :: 5 - Production/Stable',
		'Intended Audience :: Developers',
		'Topic :: Software Development :: Build Tools',
		'License :: OSI Approved :: MIT License',
		'Programming Language :: Python :: 3.6'
	]
)